cmake_minimum_required(VERSION 3.10)

if (TARGET Tasmanian::Tasmanian)
    return() # exit if Tasmanian has already been found and defined
endif()


####### Expanded from @PACKAGE_INIT@ by configure_package_config_file() #######
####### Any changes to this file will be overwritten by the next CMake run ####
####### The input file was TasmanianConfig.in.cmake                            ########

get_filename_component(PACKAGE_PREFIX_DIR "${CMAKE_CURRENT_LIST_DIR}/../../" ABSOLUTE)

macro(set_and_check _var _file)
  set(${_var} "${_file}")
  if(NOT EXISTS "${_file}")
    message(FATAL_ERROR "File or directory ${_file} referenced by variable ${_var} does not exist !")
  endif()
endmacro()

macro(check_required_components _NAME)
  foreach(comp ${${_NAME}_FIND_COMPONENTS})
    if(NOT ${_NAME}_${comp}_FOUND)
      if(${_NAME}_FIND_REQUIRED_${comp})
        set(${_NAME}_FOUND FALSE)
      endif()
    endif()
  endforeach()
endmacro()

####################################################################################

# https://cmake.org/cmake/help/v3.5/module/CMakePackageConfigHelpers.html#module:CMakePackageConfigHelpers
# seems to indicate that I need to pre-pend PACKAGE_ to the variable names after the 
####### Expanded from @PACKAGE_INIT@ by configure_package_config_file() #######
####### Any changes to this file will be overwritten by the next CMake run ####
####### The input file was TasmanianConfig.in.cmake                            ########

get_filename_component(PACKAGE_PREFIX_DIR "${CMAKE_CURRENT_LIST_DIR}/../../" ABSOLUTE)

macro(set_and_check _var _file)
  set(${_var} "${_file}")
  if(NOT EXISTS "${_file}")
    message(FATAL_ERROR "File or directory ${_file} referenced by variable ${_var} does not exist !")
  endif()
endmacro()

macro(check_required_components _NAME)
  foreach(comp ${${_NAME}_FIND_COMPONENTS})
    if(NOT ${_NAME}_${comp}_FOUND)
      if(${_NAME}_FIND_REQUIRED_${comp})
        set(${_NAME}_FOUND FALSE)
      endif()
    endif()
  endforeach()
endmacro()

#################################################################################### macro
# but this doesn't seem to work, not sure if this is a "relocatable package" (low concern)
include("/home/piwheels/.local/lib/Tasmanian/Tasmanian.cmake")

if ("OFF" AND NOT TARGET MPI::MPI_CXX)
    if (NOT MPI_CXX_COMPILER)
        set(MPI_CXX_COMPILER "")
    endif()
    if ("OFF" AND NOT MPI_Fortran_COMPILER)
        set(MPI_Fortran_COMPILER "")
    endif()
    find_package(MPI REQUIRED)
endif()

add_executable(Tasmanian::tasgrid IMPORTED)
set_property(TARGET Tasmanian::tasgrid PROPERTY IMPORTED_LOCATION "/home/piwheels/.local/bin/tasgrid${CMAKE_EXECUTABLE_SUFFIX_CXX}")

add_library(Tasmanian::Tasmanian INTERFACE IMPORTED GLOBAL) # master target
target_link_libraries(Tasmanian::Tasmanian INTERFACE Tasmanian_master)

if (ON)
    set(Tasmanian_SHARED_FOUND "ON")
    add_library(Tasmanian::shared INTERFACE IMPORTED GLOBAL) # master target
    target_link_libraries(Tasmanian::shared INTERFACE Tasmanian_master)
else()
    set(Tasmanian_STATIC_FOUND "ON")
    add_library(Tasmanian::static INTERFACE IMPORTED GLOBAL) # master target
    target_link_libraries(Tasmanian::static INTERFACE Tasmanian_master)

    if (OFF)
    # Since Tasmanian does not transitively include <cuda.h> and since all CUDA calls are wrapped in CXX API,
    # projects do not require cuda language to link to Tasmanian; however, CMake adds the extraneous dependence.
    # If Tasmanian was build with CUDA and if the user has not explicitly enabled the CUDA language,
    # then overwrite the CMake generated extraneous CUDA requirements and link with the CXX compiler only.
    # This hack is not necessary when building shared libraries.
        if (NOT CMAKE_CUDA_COMPILER)
            set_target_properties(Tasmanian_libsparsegrid PROPERTIES IMPORTED_LINK_INTERFACE_LANGUAGES_DEBUG "CXX")
            set_target_properties(Tasmanian_libsparsegrid PROPERTIES IMPORTED_LINK_INTERFACE_LANGUAGES_RELEASE "CXX")
            get_target_property(_TasLibs Tasmanian_libsparsegrid INTERFACE_LINK_LIBRARIES)
            if (${CMAKE_SYSTEM_NAME} STREQUAL "Windows")
                set_target_properties(Tasmanian_libsparsegrid PROPERTIES INTERFACE_LINK_LIBRARIES "${_TasLibs};")
            else()
                set_target_properties(Tasmanian_libsparsegrid PROPERTIES INTERFACE_LINK_LIBRARIES "${_TasLibs};;dl")
            endif()
            unset(_TasLibs)
        endif()
    endif()
endif()

if (OFF)
    if (NOT "" STREQUAL "")
        list(APPEND CMAKE_PREFIX_PATH "")
    endif()
    list(APPEND CMAKE_PREFIX_PATH "")
    foreach(_tsg_package hip rocblas rocsparse rocsolver )
        find_package(${_tsg_package} REQUIRED)
    endforeach()
endif()

if (TARGET tasmanian) # swig Tasmanian Fortran
    add_library(Tasmanian_libfortran03 INTERFACE IMPORTED GLOBAL)
    target_link_libraries(Tasmanian_libfortran03 INTERFACE tasmanian)
endif()

if (TARGET Tasmanian_libfortran03)
    add_library(Tasmanian::Fortran INTERFACE IMPORTED GLOBAL)
    target_link_libraries(Tasmanian::Fortran INTERFACE Tasmanian_libfortran90 Tasmanian_libfortran03)

    if (TARGET Tasmanian_libfortranmpi03)
        target_link_libraries(Tasmanian::Fortran INTERFACE Tasmanian_libfortranmpi03)
    endif()

    set(Tasmanian_FORTRAN_FOUND "ON")
endif()

# export the python path so other projects can configure python scripts
if (ON)
    set_and_check(Tasmanian_PYTHONPATH "/home/piwheels/.local/share/Tasmanian/python/")
    set(Tasmanian_PYTHON_FOUND "ON")
endif()

# export the MATLAB paths so other projects can write files directly to MATLAB
if (NOT "" STREQUAL "")
    set_and_check(Tasmanian_MATLAB_WORK_FOLDER "")
    set_and_check(Tasmanian_MATLABPATH "/home/piwheels/.local/share/Tasmanian/matlab/")
    set(Tasmanian_MATLAB_FOUND "ON")
endif()

set(Tasmanian_OPENMP_FOUND "TRUE")
set(Tasmanian_BLAS_FOUND   "ON")
set(Tasmanian_MPI_FOUND    "OFF")
set(Tasmanian_CUDA_FOUND   "OFF")
set(Tasmanian_HIP_FOUND    "OFF")
set(Tasmanian_DPCPP_FOUND  "OFF")
set(Tasmanian_MAGMA_FOUND  "OFF")

# component GPU uses either GPU backend (CUDA or HIP)
if (Tasmanian_CUDA_FOUND OR Tasmanian_HIP_FOUND OR Tasmanian_DPCPP_FOUND)
    set(Tasmanian_GPU_FOUND "ON")
endif()

# write component info
foreach(_tsg_comp ${Tasmanian_FIND_COMPONENTS})
    if (Tasmanian_${_tsg_comp}_FOUND)
        message(STATUS "Tasmanian component ${_tsg_comp}: found")
    else()
        if (Tasmanian_FIND_REQUIRED_${_tsg_comp})
            message(WARNING "Tasmanian required component ${_tsg_comp}: missing (error)")
        else()
            message(STATUS "Tasmanian optional component ${_tsg_comp}: missing")
        endif()
    endif()
endforeach()
unset(_tsg_comp)

check_required_components(Tasmanian)
